@@ -87,6 +87,7 @@ the \l Qt::LeftButton and \l Qt::RightButton enumeration values as \c Qt.LeftBut
 
 
 \section1 Types
+\target globalqtobjecttypes
 
 The Qt object also contains helper functions for creating objects of specific
 data types. This is primarily useful when setting the properties of an item
@@ -109,8 +110,6 @@ creating objects of specific data types are also available for clients to use:
 \li \c matrix4x4 - use \l{Qt::matrix4x4()}{Qt.matrix4x4()}
 \endlist
 
-There are also string based constructors for these types. See \l{qtqml-typesystem-valuetypes.html}{QML Value Types} for more information.
-
 \section1 Date/Time Formatters
 
 The Qt object contains several functions for formatting QDateTime, QDate and QTime values.
@@ -1300,10 +1299,15 @@ Each object in this array has the members \c lineNumber, \c columnNumber, \c fil
 For example, if the above snippet had misspelled color as 'colro' then the array would contain an object like the following:
 { "lineNumber" : 1, "columnNumber" : 32, "fileName" : "dynamicSnippet1", "message" : "Cannot assign to non-existent property \"colro\""}.
 
-Note that this function returns immediately, and therefore may not work if
+\note This function returns immediately, and therefore may not work if
 the \a qml string loads new components (that is, external QML files that have not yet been loaded).
 If this is the case, consider using \l{QtQml::Qt::createComponent()}{Qt.createComponent()} instead.
 
+\warning This function is extremely slow since it has to compile the passed QML string every time
+it is invoked. Furthermore, it's very easy to produce invalid QML when programmatically constructing
+QML code. It's much better to keep your QML components as separate files and add properties and
+methods to customize their behavior than to produce new components by string manipulation.
+
 See \l {Dynamic QML Object Creation from JavaScript} for more information on using this function.
 */
 QObject *QtObject::createQmlObject(const QString &qml, QObject *parent, const QUrl &url) const
@@ -1472,7 +1476,7 @@ Returns a \l Component object created for the type specified by \a moduleUri and
 import QtQuick
 QtObject {
     id: root
-    property Component myComponent: Qt.createComponent(Rectangle, root)
+    property Component myComponent: Qt.createComponent("QtQuick", "Rectangle", Component.Asynchronous, root)
 }
 \endqml
 This overload mostly behaves as the \c url based version, but can be used
@@ -1749,19 +1753,21 @@ enum ConsoleLogTypes {
 static QString jsStack(QV4::ExecutionEngine *engine) {
     QString stack;
 
-    QVector<QV4::StackFrame> stackTrace = engine->stackTrace(10);
-
-    for (int i = 0; i < stackTrace.size(); i++) {
-        const QV4::StackFrame &frame = stackTrace.at(i);
-
+    int i = 0;
+    for (CppStackFrame *f = engine->currentStackFrame; f && i < 10; f = f->parentFrame(), ++i) {
         QString stackFrame;
-        if (frame.column >= 0) {
-            stackFrame = QStringLiteral("%1 (%2:%3:%4)").arg(
-                frame.function, frame.source,
-                QString::number(qAbs(frame.line)), QString::number(frame.column));
+
+        if (f->isJSTypesFrame() && static_cast<JSTypesStackFrame *>(f)->isTailCalling()) {
+            stackFrame = QStringLiteral("[elided tail calls]");
         } else {
-            stackFrame = QStringLiteral("%1 (%2:%3)").arg(
-                frame.function, frame.source, QString::number(qAbs(frame.line)));
+            const int line = f->lineNumber();
+            if (line != f->missingLineNumber()) {
+                stackFrame = QStringLiteral("%1 (%2:%3)").arg(
+                        f->function(), f->source(), QString::number(qAbs(line)));
+            } else {
+                stackFrame = QStringLiteral("%1 (%2)").arg(
+                        f->function(), f->source());
+            }
         }
 
         if (i)
@@ -1988,7 +1994,7 @@ ReturnedValue ConsoleObject::method_trace(const FunctionObject *b, const Value *
     QV4::CppStackFrame *frame = v4->currentStackFrame;
     QMessageLogger(frame->source().toUtf8().constData(), frame->lineNumber(),
                    frame->function().toUtf8().constData())
-        .debug("%s", qPrintable(stack));
+        .debug(v4->qmlEngine() ? lcQml : lcJs, "%s", qPrintable(stack));
 
     return QV4::Encode::undefined();
 }
@@ -2169,8 +2175,12 @@ QString GlobalExtensions::currentTranslationContext(ExecutionEngine *engine)
 
     // The first non-empty source URL in the call stack determines the translation context.
     while (frame && context.isEmpty()) {
-        if (CompiledData::CompilationUnitBase *baseUnit = frame->v4Function->compilationUnit) {
-            const auto *unit = static_cast<const CompiledData::CompilationUnit *>(baseUnit);
+        if (ExecutableCompilationUnit *unit = frame->v4Function->executableCompilationUnit()) {
+            auto translationContextIndex = unit->data->translationContextIndex();
+            if (translationContextIndex)
+                context = unit->stringAt(*translationContextIndex);
+            if (!context.isEmpty())
+                break;
             QString fileName = unit->fileName();
             QUrl url(unit->fileName());
             if (url.isValid() && url.isRelative()) {
