@@ -56,8 +56,25 @@ const QMetaObject *QQmlTypeWrapper::metaObject() const
     if (!type.isValid())
         return nullptr;
 
-    if (type.isSingleton())
-        return type.metaObject();
+    if (type.isSingleton()) {
+        auto metaObjectCandidate = type.metaObject();
+        // if the candidate is the same as te baseMetaObject, we know that
+        // we don't have an extended singleton; in that case the
+        // actual instance might be subclass of type instead of type itself
+        // so we need to query the actual object for it's meta-object
+        if (metaObjectCandidate == type.baseMetaObject()) {
+            QQmlEnginePrivate *qmlEngine = QQmlEnginePrivate::get(engine()->qmlEngine());
+            auto object = qmlEngine->singletonInstance<QObject *>(type);
+            if (object)
+                return object->metaObject();
+        }
+        /* if we instead have an extended singleton, the dynamic proxy
+           meta-object must alreday be set up correctly
+          ### TODO: it isn't, as QQmlTypePrivate::init has no way to
+                    query the object
+        */
+        return metaObjectCandidate;
+    }
 
     return type.attachedPropertiesType(QQmlEnginePrivate::get(engine()->qmlEngine()));
 }
@@ -132,25 +149,11 @@ ReturnedValue QQmlTypeWrapper::create(QV4::ExecutionEngine *engine, QObject *o,
     return w.asReturnedValue();
 }
 
-static int enumForSingleton(QV4::ExecutionEngine *v4, String *name, QObject *qobjectSingleton,
-                            const QQmlType &type, bool *ok)
+static int enumForSingleton(QV4::ExecutionEngine *v4, String *name, const QQmlType &type, bool *ok)
 {
     Q_ASSERT(ok != nullptr);
-    int value = type.enumValue(QQmlEnginePrivate::get(v4->qmlEngine()), name, ok);
-    if (*ok)
-        return value;
-
-    // ### Optimize
-    QByteArray enumName = name->toQString().toUtf8();
-    const QMetaObject *metaObject = qobjectSingleton->metaObject();
-    for (int ii = metaObject->enumeratorCount() - 1; ii >= 0; --ii) {
-        QMetaEnum e = metaObject->enumerator(ii);
-        value = e.keyToValue(enumName.constData(), ok);
-        if (*ok)
-            return value;
-    }
-    *ok = false;
-    return -1;
+    const int value = type.enumValue(QQmlEnginePrivate::get(v4->qmlEngine()), name, ok);
+    return *ok ? value : -1;
 }
 
 ReturnedValue QQmlTypeWrapper::virtualGet(const Managed *m, PropertyKey id, const Value *receiver, bool *hasProperty)
@@ -187,7 +190,7 @@ ReturnedValue QQmlTypeWrapper::virtualGet(const Managed *m, PropertyKey id, cons
                     const bool includeEnums = w->d()->mode == Heap::QQmlTypeWrapper::IncludeEnums;
                     if (includeEnums && name->startsWithUpper()) {
                         bool ok = false;
-                        int value = enumForSingleton(v4, name, qobjectSingleton, type, &ok);
+                        int value = enumForSingleton(v4, name, type, &ok);
                         if (ok)
                             return QV4::Value::fromInt32(value).asReturnedValue();
 
@@ -413,11 +416,11 @@ ReturnedValue QQmlTypeWrapper::virtualInstanceOf(const Object *typeObject, const
     if (const QObjectWrapper *objectWrapper = var.as<QObjectWrapper>())
         return instanceOfQObject(typeWrapper, objectWrapper);
 
-    if (const QMetaObject *valueTypeMetaObject
-            = QQmlMetaType::metaObjectForValueType(typeWrapper->d()->type())) {
+    const QQmlType type = typeWrapper->d()->type();
+    if (type.isValueType()) {
         if (const QQmlValueTypeWrapper *valueWrapper = var.as<QQmlValueTypeWrapper>()) {
             return QV4::Encode(QQmlMetaObject::canConvert(valueWrapper->metaObject(),
-                                                          valueTypeMetaObject));
+                                                          type.metaObjectForValueType()));
         }
 
         // We want "foo as valuetype" to return undefined if it doesn't match.
